# Generated by Django 5.0.4 on 2024-04-05 10:34

import wagtail.blocks
import wagtail.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0002_basicmodel'),
    ]

    operations = [
        migrations.AddField(
            model_name='basicmodel',
            name='content',
            field=wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))], default=[]),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='editabledraftmodel',
            name='content',
            field=wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))], default=[]),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='editablefullmodel',
            name='content',
            field=wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))], default=[]),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='editablepreviewmodel',
            name='content',
            field=wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))], default=[]),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='editablerevisionmodel',
            name='content',
            field=wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))], default=[]),
            preserve_default=False,
        ),
    ]
